/*
  ChppWorldModelDelegate.h.cpp -

  Copyright 2008 Centre National de la Recherche Scientifique (CNRS)

  Authors: Alireza Nakhaei

*/

#ifndef HPP_WORLDMODEL_DELEGATE_H
#define HPP_WORLDMODEL_DELEGATE_H

/*************************************
INCLUDE
**************************************/
#include <iostream>
#include <fstream>
#include "KineoWorks2/kwsConfig.h"
#include "KineoWorks2/kwsRdmBuilderDelegate.h"
#include "KineoWorks2/kwsNode.h"
#include "KineoController/kppPlanPathCommand.h"
#include "KineoUtility/kitNotification.h"
#include "KineoUtility/kitNotificator.h"

#include <iostream>
#include <string>
#include <sstream>
#include <ctime>


using namespace std ;
KIT_PREDEF_CLASS ( ChppWorldModelDelegate );


/*************************************
CLASS
**************************************/
/**
   \brief Implements a class which allows to follow a roadmapbuilder evolution and stop at a specific number of node


*/
class ChppWorldModelDelegate : public CkwsRdmBuilderDelegate
{

	public :

		/**
		   \brief Protected constructor
		*/
		ChppWorldModelDelegate (int inAcceptedShootThreshold);

		/**
		   \brief   Destructor.
		*/
		virtual ~ChppWorldModelDelegate ( );

		static ChppWorldModelDelegateShPtr create(int inAcceptedShootThreshold);
		/**
		   These methods are virtual methods from Cslam3DDelegate, reimplemented.
		 */
		void willStartBuilding ( const CkwsRoadmapBuilderShPtr &i_builder, CkwsPathShPtr &io_path );

		bool shouldStopBuilding ( const CkwsRoadmapBuilderConstShPtr &i_builder );

		void didAddNode ( const CkwsRoadmapBuilderConstShPtr &i_builder, const CkwsNodeConstShPtr &i_node );


	protected :

		/**
		   \brief initialization: store weak pointer to itself
		*/
		ktStatus init ( const ChppWorldModelDelegateWkPtr &inWeakPtr );


	private :
		/**
		   \brief Store weak pointer to object
		*/
		ChppWorldModelDelegateWkPtr attWeakPtr;

		/**
		   \brief Total number of nodes generated by the builder
		 */
		long int nbAcceptedShoot;

		/**
		   \brief Number of maximum acceptable node.
		 */
		int attAcceptedShootThreshold;



};
#endif
