/*
  hppNextBestViewPlanner.h.cpp -

  Copyright 2008 Centre National de la Recherche Scientifique (CNRS)

  Authors: Alireza Nakhaei,

*/
/*****************************************
 INCLUDES
*******************************************/

#include <iostream>
#include "stdio.h"
# include <set>
#include "KineoWorks2/kwsNode.h"
#include "KineoWorks2/kwsSMLinear.h"
#include "KineoWorks2/kwsRoadmap.h"
#include "KineoWorks2/kwsRoadmap.h"

#include "KineoWorks2/kwsPathOptimizer.h"
#include "KineoWorks2/kwsRoadmapBuilder.h"
#include "KineoWorks2/kwsConfig.h"
#include "KineoWorks2/kwsDirectPath.h"
#include "KineoWorks2/kwsSMLinear.h"

#include "KineoModel/kppRotationJointComponent.h"
#include "KineoModel/kppFreeFlyerJointComponent.h"
#include "KineoModel/kppAnchorJointComponent.h"
#include "KineoModel/kppJointComponent.h"
#include "KineoModel/kppSolidComponentRef.h"
#include "KineoModel/kppConfigComponent.h"

#include "KineoKCDModel/kppKCDPolyhedron.h"
#include "KineoKCDModel/kppKCDAssembly.h"
#include "KineoWorks2/kwsPickerBasic.h"
#include "KineoWorks2/kwsPickerSmallestTree.h"
#include "KineoWorks2/kwsShooterConfigSpace.h"
#include "KineoWorks2/kwsShooterConfigList.h"
#include "KineoWorks2/kwsShooterAdaptiveMulti.h"
#include "KineoWorks2/kwsIPPRdmBuilder.h"
#include "KineoWorks2/kwsDof.h"
#include "KineoWorks2/kwsJoint.h"
#include "KineoWorks2/kwsJointDof.h"

#include "KineoWorks2/kwsLoopOptimizer.h"
#include "KineoWorks2/kwsRandomOptimizer.h"
#include "KineoWorks2/kwsDof.h"

#include "hppWorldModelPlanner/hppNextBestViewPlanner.h"
#include "hppWorldModelPlanner/kwsPlusShooter.h"


// Select verbosity at configuration by setting CXXFLAGS="... -DDEBUG=[1 or 2]"
#if DEBUG==2
#define ODEBUG2(x) std::cerr << "ChppNextBestViewPlanner:" << x << std::endl
#define ODEBUG1(x) std::cerr << "ChppNextBestViewPlanner:" << x << std::endl
#elif DEBUG==1
#define ODEBUG2(x)
#define ODEBUG1(x) std::cerr << "ChppNextBestViewPlanner:" << x << std::endl
#else
#define ODEBUG2(x)
#define ODEBUG1(x)
#endif


unsigned int ChppNextBestViewPlanner::nbObject = 0;

ChppNextBestViewPlanner::ChppNextBestViewPlanner ( )
{
	Tps = ( struct timeval* ) malloc ( sizeof ( struct timeval ) );
	Tpf = ( struct timeval* ) malloc ( sizeof ( struct timeval ) );
	Tzp = 0;
	nbObject++;
	ODEBUG2 ( "Constructor of class ChppNextBestViewPlanner. Number of objects: "<< nbObject );
}
// =========================================================================================
ChppNextBestViewPlanner::~ChppNextBestViewPlanner()
{
	free ( Tps );
	free ( Tpf );
	nbObject--;
	ODEBUG2 ( "----------------------------------------------------------------------------------------------" );
	ODEBUG2 ( "          Destructor of class ChppNextBestViewPlanner. Number of objects: "<< nbObject );
	ODEBUG2 ( "----------------------------------------------------------------------------------------------" );
}
// =========================================================================================
ktStatus ChppNextBestViewPlanner::initializeProblem ( CkwsDeviceShPtr inDevice )
{
	ODEBUG2 ( ":initializeProblem:..." );
	//attDevice = KIT_DYNAMIC_PTR_CAST ( CkppDeviceComponent , inDevice->clone() );
	CkppDeviceComponentShPtr robotBox = KIT_DYNAMIC_PTR_CAST ( CkppDeviceComponent , inDevice->clone() );
	if ( !robotBox )
	{
		ODEBUG2 ( ":initializeProblem: ERROR No robotBox." );
		return KD_ERROR;
	}
 	addHppProblemAtBeginning ( robotBox, 0.01 );
	
 	ODEBUG2 ( "TEST : colon" );
 	ODEBUG2 ( "TEST :  NUMBER OF DOF = " << robotBox-> countDofs() );
 	ODEBUG2 ( "TEST : NUMBER OF EXTRA DOF = " << robotBox-> countExtraDofs() );

	CkwsSMLinearShPtr steeringMethod = CkwsSMLinear::create();
	if ( !steeringMethod )
	{
		ODEBUG1 ( "initializeProblem: Error No steeringMethod." );
		return KD_ERROR;
	}
	robotBox->steeringMethod(steeringMethod);
	/*
	steeringMethodIthProblem ( 0, steeringMethod );

	CkwsRoadmapShPtr kwsRoadmap = CkwsRoadmap::create ( robotBox ) ;
	if ( !kwsRoadmap )
	{
		ODEBUG1 ( "initializeProblem: Error No kwsPlusRoadmap" );
		return KD_ERROR;
	}

	CkwsDistanceShPtr distance_evaluator = CkwsDistance::create();
	if ( !distance_evaluator )
	{
		ODEBUG1 ( "initializeProblem: Error No distance_evaluator" );
		return KD_ERROR;
	}
	robotBox->distance ( distance_evaluator );

	CkwsPickerBasicShPtr picker = CkwsPickerBasic::create ();
	CkwsPlusShooterShPtr shooterCfgSp =  CkwsPlusShooter::create();
	if ( !shooterCfgSp )
	{
		ODEBUG1 ( "initializeProblem: Error No shooterCfgSp" );
		return KD_ERROR;
	}

	CkwsIPPRdmBuilderShPtr kwsRoadmapBuilder = CkwsIPPRdmBuilder::create ( kwsRoadmap, 0.01,distance_evaluator,picker, shooterCfgSp );
	if ( !kwsRoadmapBuilder )
	{
		ODEBUG1 ( "initializeProblem: Error No kwsRoadmapBuilder" );
		return KD_ERROR;
	}
	roadmapBuilderIthProblem ( 0, kwsRoadmapBuilder );*/

	// clear upstacle list
	std::vector<CkcdObjectShPtr> listOfObstacle;
	obstacleList ( listOfObstacle );
	ODEBUG2 ( "Initializeing the NextBestView Planner was done. " );
	return KD_OK;
}
//==========================================================================================================
ktStatus ChppNextBestViewPlanner::setCameraParameters ( double inWidthAngle , double inHeightAngle,double inObservableDistance )
{
	ODEBUG2 ( "setCameraParameters ..." );
	att3DGridMap.setCameraParameters ( inWidthAngle ,inHeightAngle,inObservableDistance );
	ODEBUG2 ( "setCameraParameters end." );
	return KD_OK;
}
//==========================================================================================================
ktStatus ChppNextBestViewPlanner::setCoveringAngle ( double inHeightAngleMin, double inHeightAngleMax, double inWidthAngleMin, double inWidthAngleMax )
{
	ODEBUG2 ( "setCoveringAngle ..." );
	att3DGridMap.setCoveringAngle ( inHeightAngleMin, inHeightAngleMax,inWidthAngleMin, inWidthAngleMax );
	ODEBUG2 ( "setCoveringAngle end." );
	return KD_OK;
}
// =========================================================================================
ktStatus ChppNextBestViewPlanner::setResolution ( double inResolution )
{
	att3DGridMap.setResolution ( inResolution );
	return KD_OK;
}
// =========================================================================================
ktStatus ChppNextBestViewPlanner::allocateMap ( double xstart, double ystart, double zstart,double xend, double yend, double zend )
{
	ODEBUG2 ( "allocateMap: ...." );

	att3DGridMap.allocateMap ( xstart, ystart, zstart, xend, yend, zend );

	CkppDeviceComponentShPtr robotBox = robotIthProblem ( 0 );
	if ( !robotBox )
	{
		ODEBUG1 ( ":allocateMap : No robotBox" );
		return KD_ERROR;
	}

	CkppJointComponentShPtr kppJoint = robotBox->rootJointComponent();
	if ( !kppJoint )
	{
		ODEBUG1 ( "allocateMap: bounding box robot has no root joint." );
		return KD_ERROR;
	}

	CkwsJointShPtr kwsJoint = kppJoint->kwsJoint();
	if ( kwsJoint->countDofs() != 6 )
	{
		ODEBUG1 ( "allocateMap: Root joint of bounding box is not freeflyer." );
		return KD_ERROR;
	}

	kwsJoint->dof ( 0 )->isBounded ( true );
	kwsJoint->dof ( 0 )->bounds ( xstart, xend );
	kwsJoint->dof ( 1 )->isBounded ( true );
	kwsJoint->dof ( 1 )->bounds ( ystart, yend );
	kwsJoint->dof ( 2 )->isLocked ( true );
	kwsJoint->dof ( 3 )->isLocked ( true );
	kwsJoint->dof ( 4 )->isLocked ( true );
	kwsJoint->dof ( 5 )->isBounded ( true );
	kwsJoint->dof ( 5 )->bounds ( -M_PI, M_PI );
	ODEBUG2 ( "allocateMap: was done" );

	return KD_OK;
}
// =========================================================================================
// std::vector<double> ChppNextBestViewPlanner::nextBextView ( double initX, double initY, double initTheta,double goalX, double goalY, double goalTheta )
// {
// 	ODEBUG2 ( "nextBextView ..." );
// 	ODEBUG2 ( "nextBextView ..." << " , " <<  initX<< " , " <<  initY<< " , " <<  initTheta<< " , " <<  goalX<< " , " << goalY<< " , " <<  goalTheta );
// 	if ( !roadmapBuilderIthProblem ( 0 ) )
// 		ODEBUG1 ( ":nextBextView No roadmap builder defined" );
//
// 	roadmapBuilderIthProblem ( 0 )->roadmap()->clear();
// 	roadmapBuilderIthProblem ( 0 )->resetStartNodes();
// 	std::vector<double> vecNextBestView;
// 	vecNextBestView.resize ( 3 );
// 	std::vector<double> vecConfig;
// 	double temp = 0;
//
// 	double distanceMax= sqrt ( pow ( goalX-initX, 2 ) +pow ( goalY-initY,2 ) );
// 	ODEBUG2 ( "nextBextView ...distance = " << distanceMax );
//
// 	CkppDeviceComponentShPtr robotBox = robotIthProblem ( 0 );
// 	if ( !robotBox )
// 		ODEBUG1 ( "nextBextView :No Device" );
//
// 	CkwsPathShPtr kwsPath = CkwsPath::create ( robotBox );
//
// 	std::vector<double> dofValueInit;
// 	dofValueInit.push_back ( initX ); // X
// 	dofValueInit.push_back ( initY ); // Y
// 	dofValueInit.push_back ( 0.0 ); // Z
// 	dofValueInit.push_back ( 0.0 ); // RX
// 	dofValueInit.push_back ( 0.0 ); // RY
// 	dofValueInit.push_back ( initTheta ); //RZ
// 	CkwsConfigShPtr initConfig = CkwsConfig::create(robotBox , dofValueInit) ;
// 	if ( !initConfig )
// 		ODEBUG1 ( "nextBextView :No initConfig" );
//
// 	std::vector<double> testVectorInit;
// 	initConfig->getDofValues(testVectorInit);
// 	std::cout << "Test Init =" ;
// 	for (unsigned int i=0; i< testVectorInit.size() ; i++)
// 		cout <<  testVectorInit[i] << " , " ;
// 	cout << endl;
//
// 	ODEBUG2 ( "TEST : " );
// 	ODEBUG2 ( "TEST : NUMBER OF DOF = " << robotBox-> countDofs());
// 	ODEBUG2 ( "TEST : NUMBER OF EXTRA DOF = " << robotBox-> countExtraDofs());
//
// 	std::vector<double> dofValueGoal;
// 	dofValueGoal.push_back ( goalX ); // X
// 	dofValueGoal.push_back ( goalY ); // Y
// 	dofValueGoal.push_back ( 0.0 ); // Z
// 	dofValueGoal.push_back ( 0.0 ); // RX
// 	dofValueGoal.push_back ( 0.0 ); // RY
// 	dofValueGoal.push_back ( goalTheta ); //RZ
// 	CkwsConfigShPtr goalConfig = CkwsConfig::create(robotBox , dofValueGoal) ;
// 	if ( !goalConfig )
// 		ODEBUG1 ( "nextBextView No initConfig" );
// 	std::vector<double> testVectorGoal;
// 	goalConfig->getDofValues(testVectorGoal);
// 	std::cout << "Test Goal =" ;
// 	for (unsigned int i=0; i< testVectorGoal.size() ; i++)
// 		cout <<  testVectorGoal[i] << " , " ;
// 	cout << endl;
//
// 	CkwsDistanceShPtr distance_evaluator = CkwsDistance::create ();
// 	if ( !distance_evaluator )
// 		ODEBUG1 ( "Error No distance_evaluator" );
//
// //----------------------------
// 	/*
// 	Test that configurations are valid
// 	*/
// 	ktStatus                    success = KD_OK;
// 	bool                        solved;
// 	bool                        found;
// 	CkwsNodeConstShPtr          connectedStart;
// 	CkwsNodeConstShPtr          connectedGoal;
// 	std::list< CkwsEdgeShPtr >  unusedEdges;
//
// 	roadmapBuilderIthProblem ( 0 )-> addStartNode ( roadmapBuilderIthProblem ( 0 )->roadmapNode ( * ( initConfig.get() ) ) );
//
// 	// First call prepareSolve().
// 	success = roadmapBuilderIthProblem ( 0 )->prepareSolve ( kwsPath );
//
// 	// You must not call advanceSolve() if prepareSolve() returned KD_ERROR.
// 	if ( KD_OK == success )
// 	{
// 		bool t = true ;
// 		for ( int i=0; i< 200; i++ )
// 		{
// 			if ( !roadmapBuilderIthProblem ( 0 )->advanceSolve ( success ) )
// 				cout << "Error in advance solve : " << i << endl;
// 			if ( ( KD_OK == success ) && t )
// 			{
// 				cout << "Problem solved at iteration : " << i << endl;
// 				t = false;
// 			}
// 		}
// 		ODEBUG2 ( "number of node in Roadmap befor adding the goal = " << roadmapBuilderIthProblem ( 0 )->roadmap()->countNodes() );
//
// 		std::vector<CkwsNodeShPtr> nodeList;
// 		nodeList=nearestNodes ( goalConfig,50 );
//
// 		if ( nodeList.size() > 0 )
// 		{
// 			for ( unsigned int j = 0 ; j< nodeList.size(); j++ )
// 			{
// 				CkwsConfig config = nodeList.at ( j )->config();
// 				config.getDofValues ( vecConfig );
// 				double distance=distance_evaluator->distance ( *goalConfig, config );
// 				double visibility= att3DGridMap.getVisibility ( vecConfig[0], vecConfig[1], vecConfig[5],  goalX,  goalY,  goalTheta );
//
// 				double angle= abs ( atan2 ( ( goalY-vecConfig[1] ), ( goalX-vecConfig[0] ) ) - vecConfig[5] );
// 				double distanceToGoal= sqrt ( pow ( ( goalY-vecConfig[1] ),2 ) +pow ( ( goalX-vecConfig[0] ),2 ) );
// 				double tool=visibility* ( distanceMax/ ( .5+distanceToGoal ) +.5/ ( 1+angle*20/M_PI ) );
// 				ODEBUG2 ( "visibility for ( " << vecConfig[0] << " , "  << vecConfig[1] << " , "<<vecConfig[5]*180/M_PI << " )= " <<  visibility <<" ,Distance =  " << distance << " ,  Tool= "<< tool << ", angle= "<< angle*180/M_PI << ", distanceToGoal= " << distanceToGoal );
// 				if ( visibility>=0 )
// 				{
// 					if ( tool>temp )
// 					{
// 						temp=tool;
// 						vecNextBestView[0]=vecConfig[0];
// 						vecNextBestView[1]=vecConfig[1];
// 						vecNextBestView[2]=vecConfig[5];
// 					}
// 				}
// 			}
// 			ODEBUG2 ( "Next bext view point is = " <<  vecNextBestView[0] << " , " << vecNextBestView[1] << " , " << vecNextBestView[2] );
// 		}
//
// 	}
// 	success = roadmapBuilderIthProblem ( 0 )->finishSolve ( kwsPath, success );
// 	ODEBUG2 ( "End of nextBextView" );
// 	return vecNextBestView;
// }
/*
// =========================================================================================
std::vector<double> ChppNextBestViewPlanner::nextBextView ( double initX, double initY, double initTheta,double goalX, double goalY, double goalTheta )
{
	ODEBUG2 ( "nextBextView ..." );
	ODEBUG2 ( "nextBextView ..." << " , " <<  initX<< " , " <<  initY<< " , " <<  initTheta<< " , " <<  goalX<< " , " << goalY<< " , " <<  goalTheta );
	if ( !roadmapBuilderIthProblem ( 0 ) )
		ODEBUG1 ( ":nextBextView No roadmap builder defined" );

	roadmapBuilderIthProblem ( 0 )->roadmap()->clear();
	roadmapBuilderIthProblem ( 0 )->resetStartNodes();
	std::vector<double> vecNextBestView;
	vecNextBestView.resize ( 3 );
	std::vector<double> vecConfig;
	double temp = 0;

	double distanceMax= sqrt ( pow ( goalX-initX, 2 ) +pow ( goalY-initY,2 ) );
	ODEBUG2 ( "nextBextView ...distance = " << distanceMax );

	CkppDeviceComponentShPtr robotBox = robotIthProblem ( 0 );
	if ( !robotBox )
		ODEBUG1 ( "nextBextView :No Device" );

	CkwsPathShPtr kwsPath = CkwsPath::create ( robotBox );

	std::vector<double> dofValueInit;
	dofValueInit.push_back ( initX ); // X
	dofValueInit.push_back ( initY ); // Y
	dofValueInit.push_back ( 0.0 ); // Z
	dofValueInit.push_back ( 0.0 ); // RX
	dofValueInit.push_back ( 0.0 ); // RY
	dofValueInit.push_back ( initTheta ); //RZ
	CkwsConfigShPtr initConfig = CkwsConfig::create ( robotBox , dofValueInit ) ;
	if ( !initConfig )
		ODEBUG1 ( "nextBextView :No initConfig" );

	std::vector<double> testVectorInit;
	initConfig->getDofValues ( testVectorInit );
	std::cout << "Test Init =" ;
	for ( unsigned int i=0; i< testVectorInit.size() ; i++ )
		cout <<  testVectorInit[i] << " , " ;
	cout << endl;

	ODEBUG2 ( "TEST : " );
	ODEBUG2 ( "TEST : NUMBER OF DOF = " << robotBox-> countDofs() );
	ODEBUG2 ( "TEST : NUMBER OF EXTRA DOF = " << robotBox-> countExtraDofs() );

	std::vector<double> dofValueGoal;
	dofValueGoal.push_back ( goalX ); // X
	dofValueGoal.push_back ( goalY ); // Y
	dofValueGoal.push_back ( 0.0 ); // Z
	dofValueGoal.push_back ( 0.0 ); // RX
	dofValueGoal.push_back ( 0.0 ); // RY
	dofValueGoal.push_back ( goalTheta ); //RZ
	CkwsConfigShPtr goalConfig = CkwsConfig::create ( robotBox , dofValueGoal ) ;
	if ( !goalConfig )
		ODEBUG1 ( "nextBextView No initConfig" );
	std::vector<double> testVectorGoal;
	goalConfig->getDofValues ( testVectorGoal );
	std::cout << "Test Goal =" ;
	for ( unsigned int i=0; i< testVectorGoal.size() ; i++ )
		cout <<  testVectorGoal[i] << " , " ;
	cout << endl;

	CkwsDistanceShPtr distance_evaluator = CkwsDistance::create ();
	if ( !distance_evaluator )
		ODEBUG1 ( "Error No distance_evaluator" );

//----------------------------
	ktStatus                    success = KD_OK;
	bool                        solved;
	bool                        found;
	CkwsNodeConstShPtr          connectedStart;
	CkwsNodeConstShPtr          connectedGoal;
	std::list< CkwsEdgeShPtr >  unusedEdges;

	roadmapBuilderIthProblem ( 0 )-> addStartNode ( roadmapBuilderIthProblem ( 0 )->roadmapNode ( * ( initConfig.get() ) ) );

	// First call prepareSolve().
	success = roadmapBuilderIthProblem ( 0 )->prepareSolve ( kwsPath );

	// You must not call advanceSolve() if prepareSolve() returned KD_ERROR.
	if ( KD_OK == success )
{
		bool t = true ;
		for ( int i=0; i< 500; i++ )
			roadmapBuilderIthProblem ( 0 )->advanceSolve ( success );
}
	success == KD_ERROR;
	success = roadmapBuilderIthProblem ( 0 )->finishSolve ( kwsPath, success );
	ODEBUG2 ( "number of node in Roadmap befor adding the goal = " << roadmapBuilderIthProblem ( 0 )->roadmap()->countNodes() );

	std::vector<CkwsNodeShPtr> nodeList;
	//nodeList=nearestNodes ( goalConfig,100 );

	for (unsigned int i=0 ; i< roadmapBuilderIthProblem ( 0 )->roadmap()->countNodes();i++)
		nodeList.push_back(roadmapBuilderIthProblem ( 0 )->roadmap()->node(i));
	if ( nodeList.size() > 0 )
{
		for ( unsigned int j = 0 ; j< nodeList.size(); j++ )
{
			CkwsConfig config = nodeList.at ( j )->config();
			config.getDofValues ( vecConfig );
			double distance=distance_evaluator->distance ( *goalConfig, config );
			double visibility= att3DGridMap.getVisibility ( vecConfig[0], vecConfig[1], vecConfig[5],  goalX,  goalY,  goalTheta );

			double angle= abs ( atan2 ( ( goalY-vecConfig[1] ), ( goalX-vecConfig[0] ) ) - vecConfig[5] );
			double distanceToGoal= sqrt ( pow ( ( goalY-vecConfig[1] ),2 ) +pow ( ( goalX-vecConfig[0] ),2 ) );
			//double tool=visibility* ( distanceMax/ ( .5+distanceToGoal ) +.5/ ( 1+angle*20/M_PI ) );
			double tool=visibility;
			ODEBUG2 ( "visibility for ( " << vecConfig[0] << " , "  << vecConfig[1] << " , "<<vecConfig[5]*180/M_PI << " )= " <<  visibility <<" ,Distance =  " << distance << " ,  Tool= "<< tool << ", angle= "<< angle*180/M_PI << ", distanceToGoal= " << distanceToGoal );
			if ( visibility>=0 )
{
				if ( tool>temp )
{
					temp=tool;
					vecNextBestView[0]=vecConfig[0];
					vecNextBestView[1]=vecConfig[1];
					vecNextBestView[2]=vecConfig[5];
}
}
}
		ODEBUG2 ( "Next bext view point is = " <<  vecNextBestView[0] << " , " << vecNextBestView[1] << " , " << vecNextBestView[2] );
}

	ODEBUG2 ( "End of nextBextView" );
	return vecNextBestView;
}
*/

// =========================================================================================
std::vector<double> ChppNextBestViewPlanner::nextBextView ( double initX, double initY, double initTheta,double goalX, double goalY, double goalTheta )
{
	ODEBUG2 ( "nextBextView ..." );
	ODEBUG2 ( "nextBextView ..." << " , " <<  initX<< " , " <<  initY<< " , " <<  initTheta<< " , " <<  goalX<< " , " << goalY<< " , " <<  goalTheta );
/*	if ( !roadmapBuilderIthProblem ( 0 ) )
		ODEBUG1 ( ":nextBextView No roadmap builder defined" );

	roadmapBuilderIthProblem ( 0 )->roadmap()->clear();
	roadmapBuilderIthProblem ( 0 )->resetStartNodes();*/
	std::vector<double> vecNextBestView;
	vecNextBestView.resize ( 3 );
	std::vector<double> vecConfig;
	double temp = 0;

	CkppDeviceComponentShPtr robotBox = robotIthProblem ( 0 );
	if ( !robotBox )
		ODEBUG1 ( "nextBextView :No Device" );

	CkwsPathShPtr kwsPath = CkwsPath::create ( robotBox );

	std::vector<double> dofValueInit;
	dofValueInit.push_back ( initX ); // X
	dofValueInit.push_back ( initY ); // Y
	dofValueInit.push_back ( 0.0 ); // Z
	dofValueInit.push_back ( 0.0 ); // RX
	dofValueInit.push_back ( 0.0 ); // RY
	dofValueInit.push_back ( initTheta ); //RZ
	
	CkwsConfigShPtr initConfig = CkwsConfig::create ( robotBox , dofValueInit ) ;
	if ( !initConfig )
		ODEBUG1 ( "nextBextView :No initConfig" );

	std::vector<double> testVectorInit;
	initConfig->getDofValues ( testVectorInit );
	std::cout << "Test Init =" ;
	for ( unsigned int i=0; i< testVectorInit.size() ; i++ )
		cout <<  testVectorInit[i] << " , " ;
	cout << endl;

	ODEBUG2 ( "nextBextView : TEST : Robot box original" );
	ODEBUG2 ( "nextBextView : TEST :  NUMBER OF DOF = " << robotBox-> countDofs() );
	ODEBUG2 ( "nextBextView : TEST : NUMBER OF EXTRA DOF = " << robotBox-> countExtraDofs() );

	std::vector<double> dofValueGoal;
	dofValueGoal.push_back ( goalX ); // X
	dofValueGoal.push_back ( goalY ); // Y
	dofValueGoal.push_back ( 0.0 ); // Z
	dofValueGoal.push_back ( 0.0 ); // RX
	dofValueGoal.push_back ( 0.0 ); // RY
	dofValueGoal.push_back ( goalTheta ); //RZ
	CkwsConfigShPtr goalConfig = CkwsConfig::create ( robotBox , dofValueGoal ) ;
	if ( !goalConfig )
		ODEBUG1 ( "nextBextView No initConfig" );
	std::vector<double> testVectorGoal;
	goalConfig->getDofValues ( testVectorGoal );
	std::cout << "Test Goal =" ;
	for ( unsigned int i=0; i< testVectorGoal.size() ; i++ )
		cout <<  testVectorGoal[i] << " , " ;
	cout << endl;

	CkwsDistanceShPtr distance_evaluator = CkwsDistance::create ();
	if ( !distance_evaluator )
		ODEBUG1 ( "Error No distance_evaluator" );
//-------------------------------------------------------------
	CkwsRoadmapShPtr kwsRoadmap = CkwsRoadmap::create ( robotBox ) ;
	if ( !kwsRoadmap )
	{
		ODEBUG1 ( "initializeProblem: Error No kwsPlusRoadmap" );
	}

	robotBox->distance ( distance_evaluator );

	CkwsPickerBasicShPtr picker = CkwsPickerBasic::create ();

	CkwsShooterConfigSpaceShPtr shooterCfgSp =  CkwsShooterConfigSpace::create();
	if ( !shooterCfgSp )
	{
		ODEBUG1 ( "initializeProblem: Error No shooterCfgSp" );
	}

	CkwsDiffusingRdmBuilderShPtr kwsRoadmapBuilder = CkwsDiffusingRdmBuilder::create ( kwsRoadmap, 0.01,distance_evaluator,picker, shooterCfgSp );
	if ( !kwsRoadmapBuilder )
	{
		ODEBUG1 ( "initializeProblem: Error No kwsRoadmapBuilder" );
	}

//----------------------------
	ktStatus                    success = KD_OK;
	bool                        solved;
	bool                        found;
	CkwsNodeConstShPtr          connectedStart;
	CkwsNodeConstShPtr          connectedGoal;
	std::list< CkwsEdgeShPtr >  unusedEdges;

	kwsRoadmapBuilder-> addStartNode ( kwsRoadmapBuilder->roadmapNode ( * ( initConfig.get() ) ) );

	// First call prepareSolve().
	success = kwsRoadmapBuilder->prepareSolve ( kwsPath );

	// You must not call advanceSolve() if prepareSolve() returned KD_ERROR.
	unsigned int nbShoot=0;
	if ( KD_OK == success )
	{
		bool t = true ;
		//for ( int i=0; i< 200; i++ )
		while (kwsRoadmapBuilder->roadmap()->countNodes()<200)
		{
			kwsRoadmapBuilder->advanceSolve ( success );
			nbShoot++;
		}
	}

	success == KD_ERROR;
	success = kwsRoadmapBuilder->finishSolve ( kwsPath, success );
	ODEBUG2 ( "number of node in Roadmap befor adding the goal = " << kwsRoadmapBuilder->roadmap()->countNodes() );

	std::vector<CkwsNodeShPtr> nodeList;

	for ( unsigned int i=0 ; i< kwsRoadmapBuilder->roadmap()->countNodes();i++ )
		nodeList.push_back ( kwsRoadmapBuilder->roadmap()->node ( i ) );
	
	if ( nodeList.size() > 0 )
	{
		for ( unsigned int j = 0 ; j< nodeList.size(); j++ )
		{
			CkwsConfig config = nodeList.at ( j )->config();
			config.getDofValues ( vecConfig );
			double visibility= att3DGridMap.getVisibility ( vecConfig[0], vecConfig[1], vecConfig[5],  goalX,  goalY,  goalTheta );
			double tool=visibility;
			ODEBUG2 ( "visibility for ( " << vecConfig[0] << " , "  << vecConfig[1] << " , "<<vecConfig[5]*180/M_PI << " )= " <<  visibility);
			if ( visibility>=0 )
			{
				if ( tool>temp )
				{
					temp=tool;
					vecNextBestView[0]=vecConfig[0];
					vecNextBestView[1]=vecConfig[1];
					vecNextBestView[2]=vecConfig[5];
				}
			}
		}
		ODEBUG2 ( "Next bext view point is = " <<  vecNextBestView[0] << " , " << vecNextBestView[1] << " , " << vecNextBestView[2] );
	}

	ODEBUG2 ( "End of nextBextView - nbShoot =" << nbShoot);
	return vecNextBestView;
}

// =========================================================================================
/*
std::vector<double> ChppNextBestViewPlanner::nextBextView ( double initX, double initY, double initTheta,double goalX, double goalY, double goalTheta )
{
	ODEBUG2 ( " : nextBextView ..." );
	ODEBUG2 ( " : nextBextView ..." << " , " <<  initX<< " , " <<  initY<< " , " <<  initTheta<< " , " <<  goalX<< " , " << goalY<< " , " <<  goalTheta );

	std::vector<CkwsConfigShPtr> configVector;
	CkppDeviceComponentShPtr robotBox = robotIthProblem ( 0 );
	if ( !robotBox )
	{
		ODEBUG1 ( " : nextBextView : ERROR No robotBox" );
	}

	for ( unsigned int i=0; i<3000 ; i++ )
	{
		CkwsConfigShPtr config= CkwsConfig::create ( robotBox );
		config->randomize();
		if ( ( robotBox->configValidators()->validate ( *config ) ) )
			configVector.push_back ( config );
	}

	std::vector<double> vecNextBestView;
	vecNextBestView.resize ( 3 );
	double temp = 0;

	for ( unsigned int j = 0 ; j< configVector.size(); j++ )
	{
		std::vector<double> vecConfig;
		configVector[j]->getDofValues ( vecConfig );

		double visibility= att3DGridMap.getVisibility ( vecConfig[0], vecConfig[1], vecConfig[5],  goalX,  goalY,  goalTheta );
		double tool=visibility;
		ODEBUG2 ( "visibility for ( " << vecConfig[0] << " , "  << vecConfig[1] << " , "<<vecConfig[5]*180/M_PI << " )= " <<  visibility );
		if ( visibility>=0 )
		{
			if ( tool>temp )
			{
				temp=tool;
				vecNextBestView[0]=vecConfig[0];
				vecNextBestView[1]=vecConfig[1];
				vecNextBestView[2]=vecConfig[5];
			}
		}
	}
	ODEBUG2 ( " nextBextView :configVector.size() = " << configVector.size());

	if(configVector.size()==0)
		ODEBUG2 ( " nextBextView :ERROR - No acceptable config for test" );

	ODEBUG2 ( "Next bext view point is = " <<  vecNextBestView[0] << " , " << vecNextBestView[1] << " , " << vecNextBestView[2] );
	ODEBUG2 ( "nextBextView End of nextBextView" );
	return vecNextBestView;
}
*/
// =========================================================================================
double ChppNextBestViewPlanner::visibility ( double inX, double inY, double inTheta, double goalX, double goalY, double goalTheta )
{
	return att3DGridMap.getVisibility ( inX, inY, inTheta, goalX, goalY, goalTheta );
}
// =========================================================================================
ktStatus ChppNextBestViewPlanner::updateRangeMap ( const std::vector<char> inRangeMap )
{
	att3DGridMap.setGridState ( inRangeMap );
	return KD_OK;
}
//==========================================================================================================
std::vector<CkwsNodeShPtr> ChppNextBestViewPlanner::nearestNodes ( CkwsConfigShPtr config, unsigned int n )
{
	std::vector<CkwsNodeShPtr> nearestNodesVec;
	std::set<double> distanceSet;
	std::map<double,CkwsNodeShPtr > mapDistanceNode;

	double distance;

	CkwsDistanceShPtr distance_evaluator = CkwsDistance::create ();
	if ( !distance_evaluator )
		ODEBUG1 ( " : nearestNodes : Error :  No distance_evaluator" );

	ODEBUG2 ( "Number of node to test : " << roadmapBuilderIthProblem ( 0 )->roadmap()->countNodes() );
	for ( unsigned int i=0; i< roadmapBuilderIthProblem ( 0 )->roadmap()->countNodes(); i++ )
	{

		CkwsConfig tempConf= roadmapBuilderIthProblem ( 0 )->roadmap()->node ( i )->config();
		distance=distance_evaluator->distance ( *config, tempConf );
		distanceSet.insert ( distance );
		mapDistanceNode[distance]=roadmapBuilderIthProblem ( 0 )->roadmap()->node ( i );
	}
	ODEBUG2 ( "DistanceSet size = " <<  distanceSet.size() );
	std::set<double>:: iterator it;
	it=distanceSet.begin();
	while ( it!= distanceSet.end() && nearestNodesVec.size() <n )
	{
		double dd= *it;
		nearestNodesVec.push_back ( mapDistanceNode.find ( dd )->second );
		it++;
	}
	ODEBUG2 ( "nearestNodesVec size = " <<  nearestNodesVec.size() );
	return nearestNodesVec;
}
