/*
  ChppWorldModelDelegate.h.cpp -

  Copyright 2008 Centre National de la Recherche Scientifique (CNRS)

  Authors: Alireza Nakhaei

*/



/*****************************************
 INCLUDES
*******************************************/

#include <iostream>
#include "hppWorldModelPlanner/hppWorldModelDelegate.h"

// Select verbosity at configuration by setting CXXFLAGS="... -DDEBUG=[1 or 2]"
#if DEBUG==2
#define ODEBUG2(x) std::cout << "ChppWorldModelDelegate:" << x << std::endl
#define ODEBUG1(x) std::cerr << "ChppWorldModelDelegate:" << x << std::endl
#elif DEBUG==1
#define ODEBUG2(x)
#define ODEBUG1(x) std::cerr << "ChppWorldModelDelegate:" << x << std::endl
#else
#define ODEBUG2(x)
#define ODEBUG1(x)
#endif


// =========================================================================================
ChppWorldModelDelegate::ChppWorldModelDelegate ( int inAcceptedShootThreshold )
{
	nbAcceptedShoot=0;
	attAcceptedShootThreshold= inAcceptedShootThreshold;
	ODEBUG2 ( "Constructor of class ChppWorldModelDelegate." );
}
// =========================================================================================
ChppWorldModelDelegate::~ChppWorldModelDelegate()
{
	ODEBUG2 ( "------------------------------------------------------------------" );
	ODEBUG2 ( "              Destructor of class ChppWorldModelDelegate." );
	ODEBUG2 ( "------------------------------------------------------------------" );
}
// =========================================================================================
ChppWorldModelDelegateShPtr ChppWorldModelDelegate::create(int inAcceptedShootThreshold)
{
	ChppWorldModelDelegate*  flatPtr = new ChppWorldModelDelegate(inAcceptedShootThreshold);
	ChppWorldModelDelegateShPtr flatShPtr ( flatPtr );
	ChppWorldModelDelegateWkPtr flatWkPtr ( flatShPtr );
	if ( flatPtr->init ( flatWkPtr ) != KD_OK ) flatShPtr.reset();

	return flatShPtr;
}
// =========================================================================================
void ChppWorldModelDelegate::willStartBuilding ( const CkwsRoadmapBuilderShPtr &i_builder, CkwsPathShPtr &io_path )
{
	ODEBUG2 ( "Start - Cslam3DDelegate::willStartBuilding  " );
	nbAcceptedShoot=0;
	CkwsRdmBuilderDelegate::willStartBuilding ( i_builder,io_path );
	ODEBUG2 ( "End - Cslam3DDelegate::willStartBuilding  " );
}
// =========================================================================================
ktStatus ChppWorldModelDelegate::init ( const ChppWorldModelDelegateWkPtr &inWeakPtr )
{
	ktStatus success = this->ChppWorldModelDelegate::init ( inWeakPtr ) ;

	if ( KD_OK == success )
	{
		attWeakPtr = inWeakPtr;
	}
	return success;
}
// =========================================================================================
bool ChppWorldModelDelegate::shouldStopBuilding ( const CkwsRoadmapBuilderConstShPtr &i_builder )
{
	if ( nbAcceptedShoot>attAcceptedShootThreshold ) {return true;}
	return CkwsRdmBuilderDelegate::shouldStopBuilding ( i_builder );
}
// =========================================================================================
void ChppWorldModelDelegate::didAddNode ( const CkwsRoadmapBuilderConstShPtr &i_builder, const CkwsNodeConstShPtr &i_node )
{
	nbAcceptedShoot++;
	CkwsRdmBuilderDelegate::didAddNode ( i_builder,i_node );
}



